/*
  Copyright 2018 Google LLC

  Use of this source code is governed by an MIT-style
  license that can be found in the LICENSE file or at
  https://opensource.org/licenses/MIT.
*/

import {initialize, GoogleAnalyticsInitializeOptions} from './initialize.js';

import './_version.js';

/**
 * @module workbox-google-analytics
 */

export {initialize, GoogleAnalyticsInitializeOptions};
